/* 
   elmo - ELectronic Mail Operator

   Copyright (C) 2003 rzyjontko

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

   ----------------------------------------------------------------------

   
*/
/****************************************************************************
 *    IMPLEMENTATION HEADERS
 ****************************************************************************/

#include "atable.h"
#include "hash.h"
#include "raddress.h"
#include "address.h"
#include "xmalloc.h"
#include "misc.h"
#include "memchunk.h"
#include "abook.h"

/****************************************************************************
 *    IMPLEMENTATION PRIVATE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE STRUCTURES / UTILITY CLASSES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION REQUIRED EXTERNAL REFERENCES (AVOID)
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE DATA
 ****************************************************************************/

static raddress_t *tables[ADDRESS_DESCRIPTORS_COUNT];

static raddress_t *destination = NULL;

/****************************************************************************
 *    INTERFACE DATA
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTION PROTOTYPES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTIONS
 ****************************************************************************/

static void
add_address (address_t *addr)
{
  raddress_add (destination, addr);
}


/****************************************************************************
 *    INTERFACE FUNCTIONS
 ****************************************************************************/


void
atable_dump (memchunk_t *chunk, int ad)
{
  int         i;
  int         size = address_count (ad);
  address_t     *addr;
  raddress_t *list;

  if (size == 0 || size == -1){
    memchunk_intdump (chunk, 0);
    return;
  }

  list        = raddress_create_size (size + 1);
  destination = list;
  address_for_all (ad, add_address);
  destination = NULL;

  memchunk_intdump (chunk, size);
  for (i = 0; i < size; i++){
    addr = list->array[i];
    addr->index = i;
    memchunk_strdump (chunk, addr->full);
  }
  raddress_destroy (list);
}


void
atable_dump_addr (memchunk_t *chunk, address_t *addr)
{
  if (addr == NULL)
    memchunk_intdump (chunk, -1);
  else
    memchunk_intdump (chunk, addr->index);
}



void
atable_dump_raddress (memchunk_t *chunk, raddress_t *ptr)
{
  int i;

  if (ptr == NULL){
    memchunk_intdump (chunk, 0);
    return;
  }
  
  memchunk_intdump (chunk, ptr->count);
  for (i = 0; i < ptr->count; i++)
    memchunk_intdump (chunk, ptr->array[i]->index);
}



int
atable_read (memchunk_t *chunk)
{
  int     i;
  int     ad   = address_open ();
  int     size = memchunk_intget (chunk);
  char   *str;
  address_t *addr;

  if (ad == -1)
    return -1;
  
  if (size == 0)
    return ad;

  atable_point (ad);
  tables[ad] = raddress_create_size (size + 1);
  for (i = 0; i < size; i++){
    str  = memchunk_strget (chunk);
    addr = address_from_string (str);
    if (addr){
      raddress_add (tables[ad], addr);
    }
    if (str)
      xfree (str);
  }
  return ad;
}



address_t *
atable_read_addr (memchunk_t *chunk, int ad)
{
  int index = memchunk_intget (chunk);

  if (ad == -1)
    return NULL;
  
  if (index == -1)
    return NULL;

  if (tables[ad] == NULL)
    return NULL;
  
  if (index > tables[ad]->count)
    return NULL;
  
  return tables[ad]->array[index];
}



raddress_t *
atable_read_raddress (memchunk_t *chunk, int ad)
{
  int         i;
  int         count = memchunk_intget (chunk);
  raddress_t *result;

  if (count == 0)
    return NULL;

  result = raddress_create_size (count + 1);
  for (i = 0; i < count; i++){
    raddress_add (result, atable_read_addr (chunk, ad));
  }
  return result;
}



void
atable_finished (int ad)
{
  if (tables[ad])
    raddress_destroy (tables[ad]);
  tables[ad] = NULL;
}



/**
 * it is a wrapper to address_point
 * after switching to this address descriptor, addresses from addressbook
 * are added
 */
void
atable_point (int ad)
{
  address_point (ad);
  abook_put_loaded ();
}


/****************************************************************************
 *    INTERFACE CLASS BODIES
 ****************************************************************************/
/****************************************************************************
 *
 *    END MODULE atable.c
 *
 ****************************************************************************/
