/* 
   elmo - ELectronic Mail Operator

   Copyright (C) 2002 bazyl
   Copyright (C) 2003 rzyjontko

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

   ----------------------------------------------------------------------

   This module creates special window with date and time.
   It's also using special structure to show elmo's work like downloading or
   sending mails.
   
*/
/****************************************************************************
 *    IMPLEMENTATION HEADERS
 ****************************************************************************/

#include <unistd.h>
#include <pthread.h>
#include <time.h>
#include <signal.h>

#include "ecurses.h"
#include "xmalloc.h"
#include "clock.h"
#include "line.h"
#include "screen.h"
#include "error.h"
#include "cmd.h"

/****************************************************************************
 *    IMPLEMENTATION PRIVATE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE STRUCTURES / UTILITY CLASSES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION REQUIRED EXTERNAL REFERENCES (AVOID)
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE DATA
 ****************************************************************************/

static pthread_t  clock_thread;
static WINDOW    *clock_win    = NULL;
static line_t    *clock_info   = NULL;

static pthread_mutex_t mutex;

/****************************************************************************
 *    INTERFACE DATA
 ****************************************************************************/

static work_t clock_action;

/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTION PROTOTYPES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTIONS
 ****************************************************************************/

static void
startup (void)
{
  sigset_t sigset;

  sigemptyset (&sigset);
  sigaddset (&sigset, SIGINT);
  sigaddset (&sigset, SIGTERM);
  sigaddset (&sigset, SIGQUIT);
  sigaddset (&sigset, SIGSEGV);
  sigaddset (&sigset, SIGFPE);
  sigaddset (&sigset, SIGABRT);

  pthread_sigmask (SIG_BLOCK, &sigset, NULL);
}





static void *
clock_thread_fun(void *win)
{
  static char buff[25];
  time_t      curtime;
  time_t      oldtime = 0;
  struct tm  *loctime;

  startup ();
  
  for(;;){

    if (!(clock_action.work)) {
      curtime = time (NULL);
      if (curtime == oldtime)
        goto jumpout;
      loctime = localtime (&curtime);
      oldtime = curtime;
      
      pthread_mutex_lock (&mutex);
      if (isendwin () == FALSE){
        strftime (buff, sizeof(buff), "    %H:%M    ", loctime);
        mvwaddstr(win, 0, 0, buff);
        strftime (buff, sizeof(buff), " %a, %e %b ", loctime);
        mvwaddstr(win, 1, 0, buff);
      }
    }
    screen_wrefresh(win);
    pthread_mutex_unlock (&mutex);

jumpout:
    pthread_mutex_lock (&mutex);
    error_hook ();
    pthread_mutex_unlock (&mutex);
    usleep(30000);
  }
}

/****************************************************************************
 *    INTERFACE FUNCTIONS
 ****************************************************************************/

void
clock_init(void)
{
  clock_action.work = 0;
  clock_win = newwin(2, 13, LINES - 2, COLS - 13);
  wbkgdset(clock_win, COLOR_PAIR(1));
  
  clock_info = line_create(12);
  
  mvwaddstr(clock_win, 0, 0, "             ");
  mvwaddstr(clock_win, 1, 0, "             ");
  pthread_create(&clock_thread, NULL, clock_thread_fun, clock_win);
  
  pthread_detach(clock_thread);
}


void
clock_free_resources(void)
{
  pthread_cancel(clock_thread);
  line_destroy(clock_info);
  delwin(clock_win);
}



void
clock_stop (void)
{
  pthread_mutex_lock (&mutex);
}


void
clock_resume (void)
{
  pthread_mutex_unlock (&mutex);
}


/****************************************************************************
 *    INTERFACE CLASS BODIES
 ****************************************************************************/
/****************************************************************************
 *
 *    END MODULE clock.c
 *
 ****************************************************************************/
