/* 
   elmo - ELectronic Mail Operator

   Copyright (C) 2002, 2003 rzyjontko

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

   ----------------------------------------------------------------------

   This is an example file to examine libraries and to play some with
   them.
   
*/
/****************************************************************************
 *    IMPLEMENTATION HEADERS
 ****************************************************************************/

#define _GNU_SOURCE 1

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>
#include <errno.h>
#include <regex.h>
#include <locale.h>

#include "token.h"
#include "xmalloc.h"
#include "hash.h"
#include "memblock.h"
#include "hmac-md5.h"
#include "md5.h"
#include "address.h"
#include "mime.h"

/****************************************************************************
 *    IMPLEMENTATION PRIVATE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
 ****************************************************************************/
#define ATEXT "([[:alpha:][:digit:]!#\\$%&\\\\\\'\\*\\+\\/\\=\\?\\^_\\`" \
              "\\{\\}\\~\\|\\-])"
/****************************************************************************
 *    IMPLEMENTATION PRIVATE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE STRUCTURES / UTILITY CLASSES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION REQUIRED EXTERNAL REFERENCES (AVOID)
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE DATA
 ****************************************************************************/

static char long_txt[] = "Baka miaa fajny biust\n"
"Ala styl, a Goka co co lubi.\n"
"Tylko u nas za jedyne $100 lij na ala@kot.org. 12 32";

/****************************************************************************
 *    INTERFACE DATA
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTION PROTOTYPES
 ****************************************************************************/
/****************************************************************************
 *    FUNCTION SUBSTITUTES
 ****************************************************************************/

char *
prepare_error_msg (int errnum, const char *fmt, va_list ap)
{
  char *result;
  char *user_msg;
  int   user_len;
  char *system_msg;
  int   system_len;
  
  user_len = vasprintf (&user_msg, fmt, ap);

  if (errnum){
    system_msg = strerror (errnum);
    system_len = strlen (system_msg);
    result     = xmalloc (user_len + 2 + system_len + 1);
    memcpy (result, user_msg, user_len);
    memcpy (result + user_len + 2, system_msg, system_len + 1);
    result[user_len]     = ':';
    result[user_len + 1] = ' ';
  }
  else {
    result = xstrdup (user_msg);
  }

  free (user_msg);
  return result;
}



void
error_critical (int status, int errnum, const char *msg)
{
  
}


void
error_ (int errnum , const char *fmt, ...)
{
  va_list  ap;
  char    *msg;

  va_start (ap, fmt);
  msg = prepare_error_msg (errnum, fmt, ap);
  va_end (ap);

  fprintf (stderr, "%s\n", msg);

  xfree (msg);
}


void
error_vmsg (const char *fmt, va_list ap)
{
  char *msg = prepare_error_msg (0, fmt, ap);

  fprintf (stderr, "%s\n", msg);
  xfree (msg);
}


void
error_regex (int errcode, regex_t *compiled)
{
  int   length = regerror (errcode, compiled, NULL, 0);
  char *buffer = xmalloc (length + 1);

  regerror (errcode, compiled, buffer, length);

  fprintf (stderr, "%s\n", buffer);
  xfree (buffer);
}


char *
ask_for_default (const char *a, const char *b)
{
  return NULL;
}

/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTIONS
 ****************************************************************************/

static int
token_test (void)
{
  int   td;
  char *word;

  token_init ();

  td = token_open (long_txt, 5, TOKEN_BAYES_WORD_RE, 1);

  if (td == -1)
    return 1;

  while (1){
    word = token_read_next (td);
    if (word == NULL)
      break;
    printf ("%s\n", word);
  }
  token_close (td);
  return 0;
}



static int
htable_test (void)
{
  htable_t *table = htable_create (10);
  entry_t  *entry;

  htable_insert (table, "ala", (void *) 0);
  htable_insert (table, "ola", (void *) 1);
  htable_insert (table, "ula", (void *) 2);
  htable_insert (table, "ela", (void *) 3);

  entry = htable_remove (table, "ola");

  printf ("ola: %d\n", (int) entry->content);

  entry = htable_lookup (table, "ola");
  if (entry){
    printf ("burak\n");
    return 1;
  }

  printf ("ok\n");

  htable_clear (table, NULL);

  entry = htable_lookup (table, "ala");
  if (entry){
    printf ("burak\n");
    return 1;
  }

  printf ("ok\n");

  htable_destroy (table, NULL);

  return 0;
}



static int
hmac_test (void)
{
  int         i;
  mime_t      *mime = mime_create (0);
  static char digest[16];
  static char text[] = "what do ya want for nothing?";
  static char key[]  = "Jefe";
  static char chalenge[] = "PDE2ODk4NDk1NDAuNTgxODQ5NUBwb2N6dGEuZm0+";
  char *decoded;
  int   len;

  hmac_md5 (text, 28, key, 4, digest);
  for (i = 0; i < 16; i++){
    printf ("%02x", (unsigned char) digest[i]);
  }
  printf ("\n");

  mime->offset_end = strlen (chalenge);
  mime->encoding   = MENC_BASE64;

  decoded = mime_decode (mime, chalenge, NULL);
  printf ("%s\n", decoded);

  decoded = mime_encode (decoded, strlen (decoded), &len, MENC_BASE64);
  decoded[len - 1] = '\0';
  printf ("%s\n", decoded);

  return 0;
}



static int
md5_test (void)
{
  int         i;
  static char digest[16];
  MD5_CTX     context;

  MD5Init (&context);
  MD5Update (&context, "a", 1);
  MD5Final (digest, &context);

  for (i = 0; i < 16; i++){
    printf ("%02x", (unsigned char) digest[i]);
  }
  printf ("\n");
  return 0;
}



static void
display_addr (address_t *a)
{
  if (a->full)
    printf (" full:        %s\n", a->full);
  if (a->email)
    printf (" email:       <%s>\n", a->email);
  if (a->name)
    printf (" name:        '%s'\n", a->name);
  if (a->initials)
    printf (" initials:    %s\n", a->initials);
  printf (" kind:        %d\n", a->flags.bits.kind);
  if (a->flags.bits.sex == SEX_MALE)
    printf (" sex:         M\n");
  else if (a->flags.bits.sex == SEX_FEMALE)
    printf (" sex:         F\n");
  printf (" abook:       %d\n", a->flags.bits.abook);
}



static int
address_test (void)
{
  struct addr *a;

  address_init ();

  printf ("%s\n", ATEXT);
  
  printf (" sizeof (struct addr) == %d\n", sizeof (struct addr));
#define STR "ala"
  printf ("%d\n", sizeof (STR));

  a = address_from_string ("rzyjontko <rzyj@op.pl>");
  display_addr (a);

  a = address_from_string ("rzyj@op.pl (rzyjontko)");
  display_addr (a);

  a = address_from_string ("rzyj@op.pl");
  display_addr (a);

  a = address_from_string ("rzyj");
  display_addr (a);

  a = address_from_string (" rzyjontko < rzyj@op.pl >");
  display_addr (a);

  a = address_from_string ("\".ala.\" ma \".kota.\" <ala@kot.org>");
  display_addr (a);
  
  a = address_from_string ("\" [ala \\\"ma\\\" kota] \" <ala@kot.org>");
  display_addr (a);

  a = address_from_string ("elmo-users@nongnu.org");
  display_addr (a);

  address_free_resources ();

  return 0;
}


/****************************************************************************
 *    INTERFACE FUNCTIONS
 ****************************************************************************/


void
usage (int status)
{
  fprintf (stderr, "exampl [x]\n"
           "  0 - this help\n"
           "  1 - token test\n"
           "  2 - hashing tables test\n"
           "  3 - hmac test\n"
           "  4 - md5 test\n"
           "  5 - address test\n");
  exit (status);
}


int
main (int argc, char **argv){
  
  setlocale (LC_ALL, "");

  if (argc > 1){
    switch (*argv[1]){

      case '0':
        usage (EXIT_SUCCESS);
        break;

      case '1':
        return token_test ();

      case '2':
        return htable_test ();

      case '3':
        return hmac_test ();

      case '4':
        return md5_test ();

      case '5':
        return address_test ();
    
      default:
        usage (EXIT_FAILURE);
        break;
    }
  }
  else {
    usage (EXIT_FAILURE);
  }

  exit (EXIT_SUCCESS);
}


/****************************************************************************
 *    INTERFACE CLASS BODIES
 ****************************************************************************/
/****************************************************************************
 *
 *    END MODULE exampl.c
 *
 ****************************************************************************/
