/* 
   elmo - ELectronic Mail Operator

   Copyright (C) 2003 rzyjontko

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

   ----------------------------------------------------------------------

   help window
   
*/
/****************************************************************************
 *    IMPLEMENTATION HEADERS
 ****************************************************************************/

#include "select.h"
#include "help.h"
#include "cmd.h"
#include "gettext.h"
#include "keymap.h"
#include "exec.h"
#include "misc.h"
#include "xmalloc.h"
#include "eprintf.h"
#include "ask.h"

/****************************************************************************
 *    IMPLEMENTATION PRIVATE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
 ****************************************************************************/

#define PREAMBLE do { if (help_win == NULL) return; } while (0)

#define HELP_WIDTH  COLS  - 2
#define HELP_HEIGHT LINES - 6

#define HELP_FMT "%012k %030f %d"

/****************************************************************************
 *    IMPLEMENTATION PRIVATE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE STRUCTURES / UTILITY CLASSES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION REQUIRED EXTERNAL REFERENCES (AVOID)
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE DATA
 ****************************************************************************/

static select_t *help_win = NULL;

/****************************************************************************
 *    INTERFACE DATA
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTION PROTOTYPES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTIONS
 ****************************************************************************/

static rstring_t *
make_introduction (void)
{
  int        i;
  char      *str = NULL;
  rstring_t *cut;
  rstring_t *result;

  switch (cmd_state_get (0)){

    case CMD_INVALID:
      str = xstrdup (_("This is a bug.  Please submit a report."));
      break;

    case CMD_LIST:
      str = xstrdup (_("\n"));
      break;

    case CMD_READ_MAIL:
      str = xstrdup (_("\n"));
      break;

    case CMD_SELECT_BOX:
      str = xstrdup (_("\n"));
      break;

    case CMD_FETCH:
      str = xstrdup (_("\n"));
      break;

    case CMD_SENDER:
      str = xstrdup (_("\n"));
      break;

    case CMD_ATTACH:
      str = xstrdup (_("\n"));
      break;

    case CMD_ABOOK:
      str = xstrdup (_("\n"));
      break;

    case CMD_ABOOK_ADD:
      str = xstrdup (_("\n"));
      break;

    case CMD_ASK:
      str = xstrdup (_("\n"));
      break;

    case CMD_HELP:
      str = xstrdup (_("This is a bug.  Please submit a report."));
      break;
  }

  cut = rstring_split_lines (str, HELP_WIDTH - 2);

  if (cut == NULL)
    return NULL;
  
  result = rstring_create_size (cut->count + 1);
  
  for (i = 0; i < cut->count; i++){
    rstring_add (result, xstrdup (cut->array[i]));
  }

  xfree (str);
  rstring_delete (cut);
  return result;
}


static rstring_t *
make_keys (void)
{
  int         i;
  char       *fmt = ask_for_default ("help_format", NULL);
  char       *key_str;
  struct key *key;
  keymap_t   *keymap = keymaps + cmd_state_get (0);
  rstring_t  *result = rstring_create ();

  rstring_add (result, xstrdup (_("Current key combinations are:")));
  
  if (fmt == NULL)
    fmt = HELP_FMT;
  
  for (i = 0; i < keymap->count; i++){
    key     = keymap->keys + i;
    key_str = eprintf_key (fmt, key);
    rstring_add (result, key_str);
  }
  
  return result;
}


static rstring_t *
make_items (void)
{
  rstring_t *result = rstring_join (make_introduction (), make_keys ());
  
  result->allocated_all = 1;
  
  return result;
}



static void
destroy_items (rstring_t *items)
{
  rstring_delete (items);
}

/****************************************************************************
 *    INTERFACE FUNCTIONS
 ****************************************************************************/

void
help_open (void)
{
  rstring_t *items;

  if (help_win)
    return;

  items = make_items ();

  help_win = select_open (items, HELP_WIDTH, HELP_HEIGHT, 1, 1, _("help"));

  if (help_win == NULL){
    destroy_items (items);
    return;
  }

  select_show (help_win);
  cmd_state_push (CMD_HELP);
}




void
help_close (void)
{
  PREAMBLE;

  destroy_items (help_win->items);
  select_close (help_win);
  
  help_win = NULL;
  
  cmd_state_pop ();
  cmd_refresh ();
}



void
help_refresh (void)
{
  PREAMBLE;

  select_show (help_win);
}


void
help_scroll_down (void)
{
  PREAMBLE;
  
  select_next (help_win);
}



void
help_scroll_up (void)
{
  PREAMBLE;
  
  select_prev (help_win);
}



void
help_prev_page (void)
{
  PREAMBLE;
  
  select_prev_page (help_win);
}



void
help_next_page (void)
{
  PREAMBLE;
  
  select_next_page (help_win);
}



void
help_beg (void)
{
  PREAMBLE;
  
  select_first (help_win);
}



void
help_end (void)
{
  PREAMBLE;
  
  select_last (help_win);
}


/****************************************************************************
 *    INTERFACE CLASS BODIES
 ****************************************************************************/
/****************************************************************************
 *
 *    END MODULE help.c
 *
 ****************************************************************************/
