/* 
   elmo - ELectronic Mail Operator

   Copyright (C) 2002 bazyl
   Copyright (C) 2003 rzyjontko

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

   ----------------------------------------------------------------------

   showing e-mail in scrolling window
   
*/
/****************************************************************************
 *    IMPLEMENTATION HEADERS
 ****************************************************************************/

#include <unistd.h>
#include <errno.h>

#include "ecurses.h"
#include "mailreader.h"
#include "screen.h"
#include "rchtype.h"
#include "status.h"
#include "error.h"
#include "xmalloc.h"
#include "attach.h"
#include "cmd.h"
#include "wrapbox.h"
#include "gettext.h"
#include "folder.h"

/****************************************************************************
 *    IMPLEMENTATION PRIVATE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
 ****************************************************************************/

#define PREAMBLE do { if (mail_win == NULL) return; } while (0)

/****************************************************************************
 *    IMPLEMENTATION PRIVATE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE STRUCTURES / UTILITY CLASSES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION REQUIRED EXTERNAL REFERENCES (AVOID)
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE DATA
 ****************************************************************************/

static int        header_view = 0;
static int        curr_line   = 0;
static WINDOW    *mail_win;
static rchtype_t *mail_lines;

static char *mail_body;

/****************************************************************************
 *    INTERFACE DATA
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTION PROTOTYPES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTIONS
 ****************************************************************************/

static void
write_lines (void)
{
  int i;
  
  if (curr_line < 0)
    curr_line = 0;
  if (curr_line >= mail_lines->count)
    curr_line = mail_lines->count - 2;
  
  wclear (mail_win);

  for (i = 0; i < LINES - 2 && i + curr_line < mail_lines->count; i++){
    mvwaddchstr (mail_win, i, 0, mail_lines->array[i + curr_line]);
  }
}



static rchtype_t *
generate_header (mail_t *mail)
{
  rchtype_t *result = rchtype_create ();
  int        i;

  rchtype_set_mask (COLOR_PAIR (5));
  if (mail->date_str)
    rchtype_add_string (result, "Date: %s", mail->date_str);
  if (mail->from)
    rchtype_add_string (result, "From: %s", mail->from->full);
  if (mail->subject)
    rchtype_add_string (result, "Subject: %s", mail->subject);
  if (mail->to && mail->to->count){
    if (mail->to->array[0] && mail->to->array[0]->full)
      rchtype_add_string (result, "To: %s", mail->to->array[0]->full);
    else
      rchtype_add_string (result, "To:");
    for (i = 1; i < mail->to->count; i++){
      rchtype_add_string (result, "    %s", mail->to->array[i]->full);
    }
  }
  if (mail->cc && mail->cc->count){
    if (mail->cc->array[0] && mail->cc->array[0]->full)
      rchtype_add_string (result, "Cc: %s", mail->cc->array[0]->full);
    else
      rchtype_add_string (result, "Cc:");
    for (i = 1; i < mail->cc->count; i++){
      rchtype_add_string (result, "    %s", mail->cc->array[i]->full);
    }
  }
  if (mail->bcc && mail->bcc->count){
    if (mail->bcc->array[0] && mail->bcc->array[0]->full)
      rchtype_add_string (result, "Bcc: %s", mail->bcc->array[0]->full);
    else
      rchtype_add_string (result, "Bcc:");
    for (i = 1; i < mail->bcc->count; i++){
      rchtype_add_string (result, "    %s", mail->bcc->array[i]->full);
    }
  }
  if (mail->reply_to && mail->reply_to != mail->from)
    rchtype_add_string (result, "Reply-To: %s", mail->reply_to->full);
  if (mail->mua != MUA_UNKNOWN){
    if (MUA_CONSOLE (mail->mua)){
      rchtype_set_mask (A_BOLD);
    }
    rchtype_add_string (result, "MUA: %s", mail_mua_names[mail->mua]);
    if (MUA_CONSOLE (mail->mua)){
      rchtype_unset_mask (A_BOLD);
    }
  }
  rchtype_unset_mask (COLOR_PAIR (5));
  rchtype_add_string (result, "");
  return result;
}



static int
set_mail_lines (int mime_index)
{
  mail_t    *mail   = folder_mail_selected ();
  rchtype_t *header = generate_header (mail);
  
  if (wrapbox_mail_body (mail, mime_index, &mail_body, NULL) && errno){
    error_ (errno, _("impossible to display mail contents"));
    return 1;
  }

  if (mail_lines){
    rchtype_destroy (mail_lines);
    mail_lines = NULL;
  }
  
  if (mail_body){
    mail_lines = rchtype_split_lines (mail_body, COLS);
    mail_lines = rchtype_join (header, mail_lines);
  }
  else {
    mail_lines = header;
  }

  if (mail_body){
    xfree (mail_body);
    mail_body = NULL;
  }
  return 0;
}



static int
set_header_lines (void)
{
  mail_t *mail = folder_mail_selected ();

  if (wrapbox_mail_header (mail, &mail_body)){
    error_ (errno, _("impossible to display mail header"));
    return 1;
  }

  if (mail_lines){
    rchtype_destroy (mail_lines);
    mail_lines = NULL;
  }

  if (mail_body){
    mail_lines = rchtype_split_lines (mail_body, COLS);
    xfree (mail_body);
    mail_body = NULL;
  }
  return 0;
}



/****************************************************************************
 *    INTERFACE FUNCTIONS
 ****************************************************************************/

void
mailreader_line_up (void)
{
  PREAMBLE;
  
  if (curr_line == 0)
    return;
  curr_line--;
  wscrl (mail_win, -1);
  mvwaddchstr (mail_win, 0, 0, mail_lines->array[curr_line]);
  screen_wnoutrefresh (mail_win);
}



void
mailreader_line_down (void)
{
  PREAMBLE;
  
  if (curr_line >= (int)(mail_lines->count - LINES + 2))
    return;
  curr_line++;
  wscrl (mail_win, 1);
  mvwaddchstr (mail_win, LINES - 3, 0,
               mail_lines->array[curr_line + LINES - 3]);
  screen_wnoutrefresh (mail_win);
}



void
mailreader_page_up (void)
{
  PREAMBLE;
  
  curr_line -= LINES - 2;
  write_lines ();
  screen_wnoutrefresh (mail_win);
}



void
mailreader_page_down (void)
{
  PREAMBLE;
  
  curr_line += LINES - 2;
  write_lines ();
  screen_wnoutrefresh (mail_win);
}



void
mailreader_top (void)
{
  PREAMBLE;
  
  curr_line = 0;
  write_lines ();
  screen_wnoutrefresh (mail_win);
}



void
mailreader_bottom (void)
{
  PREAMBLE;
  
  curr_line = mail_lines->count - LINES + 2;
  write_lines ();
  screen_wnoutrefresh (mail_win);
}



void
mailreader_header_switch (void)
{
  PREAMBLE;

  if (! header_view){
    set_header_lines ();
    curr_line = 0;
    write_lines ();
    mailreader_refresh ();
    header_view = 1;
  }
  else {
    set_mail_lines (-1);
    curr_line  = 0;
    write_lines ();
    mailreader_refresh ();
    header_view = 0;
  }
}



void
mailreader_change_mime (void)
{
  int     mime_index = attach_selected_item;
  mail_t *mail;
  mime_t *mime;

  PREAMBLE;
  
  if (mime_index == -1)
    return;

  mail = folder_mail_selected ();
  mime = mail->rmime->array[mime_index];

  if (mime->type == MTYPE_TEXT_PLAIN ||
      mime->type == MTYPE_TEXT_OTHER ||
      mime->type == MTYPE_HTML       ||
      mime->type == MTYPE_MESSAGE){
  
    set_mail_lines (mime_index);

    curr_line  = 0;
    write_lines ();
    header_view = 0;
  }
}



void
mailreader_init (void)
{
  mail_t *mail = folder_mail_selected ();

  if (mail == NULL)
    return;
  
  mail_win   = newwin (LINES - 2, COLS, 0, 0);
  scrollok (mail_win, TRUE);
  
  if (set_mail_lines (-1))
    return;

  curr_line  = 0;
  write_lines ();

  if (mail->from)
    status_put_mailinfo (mail->from->name, mail->subject);
  else
    status_put_mailinfo (NULL, mail->subject);
  
  screen_touchwin (mail_win);
  screen_wnoutrefresh (mail_win);
  screen_doupdate ();

  header_view = 0;
  
  cmd_state_push (CMD_READ_MAIL);
}



void
mailreader_close (void)
{
  PREAMBLE;
  
  delwin(mail_win);
  rchtype_destroy (mail_lines);

  mail_lines = NULL;
  mail_win   = NULL;

  cmd_state_pop ();
}



void
mailreader_attach (void)
{
  mail_t *mail;

  PREAMBLE;
  
  mail = folder_mail_selected ();
  if (mail == NULL)
    return;
  attach_open (mail);
}



void
mailreader_refresh (void)
{
  char   *from    = NULL;
  char   *subject = NULL;
  mail_t *mail    = folder_mail_selected ();
  
  PREAMBLE;
  
  screen_touchwin (mail_win);
  screen_wnoutrefresh (mail_win);

  if (mail && mail->from)
    from = mail->from->name;
  if (mail)
    subject = mail->subject;
  
  status_put_mailinfo (from, subject);
  status_refresh ();
}

/****************************************************************************
 *    INTERFACE CLASS BODIES
 ****************************************************************************/
/****************************************************************************
 *
 *    END MODULE mailreader.c
 *
 ****************************************************************************/
