/* 
   elmo - ELectronic Mail Operator

   Copyright (C) 2002, 2003 rzyjontko

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

   ----------------------------------------------------------------------

   This is very simple module used to get full list of messages from
   file in mbox format.

*/

#define _GNU_SOURCE 1


/****************************************************************************
 *    IMPLEMENTATION HEADERS
 ****************************************************************************/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <errno.h>
#include <unistd.h>
#include <sys/stat.h>

#include "xmalloc.h"
#include "mail.h"
#include "mbox.h"
#include "hash.h"
#include "mlex.h"
#include "file.h"
#include "error.h"
#include "progress.h"
#include "gettext.h"
#include "atable.h"

/****************************************************************************
 *    IMPLEMENTATION PRIVATE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE STRUCTURES / UTILITY CLASSES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION REQUIRED EXTERNAL REFERENCES (AVOID)
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE DATA
 ****************************************************************************/
/****************************************************************************
 *    INTERFACE DATA
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTION PROTOTYPES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTIONS
 ****************************************************************************/
/****************************************************************************
 *    INTERFACE FUNCTIONS
 ****************************************************************************/


int
mbox_may_be_valid (const char *fname)
{
  struct stat st;

  if (stat (fname, &st)){
    error_ (errno, "%s", fname);
    return 0;
  }

  if (S_ISREG (st.st_mode))
    return 1;
  return 0;
}



int
mbox_mail_size (mail_t *mail)
{
  mime_t *lastmime;

  lastmime = mail->rmime->array[mail->rmime->count - 1];
  return lastmime->offset_end - mail->place.offset_header;
}
    


mail_array_t *
mbox_read_file (const char *file)
{
  struct stat   st;
  int           ret = 0;
  mail_array_t *result;

  if (stat (file, &st)){
    error_ (errno, "%s", file);
    return NULL;
  }

  if (! S_ISREG (st.st_mode) || st.st_size == 0)
    return NULL;
  
  yyin = fopen (file, "r");
  if (!yyin){
    error_ (errno, "%s", file);
    return NULL;
  }
  
  if (st.st_size){
    progress_open (_("reading file"));
    progress_set_max (st.st_size);
  }

  /* first invocation of mbox_scan tells us only if we are reading
     a mailbox; it does not bring any information yet */
  if (mlex_mbox_scan_start () != NEXT_MAIL){
    fclose (yyin);
    progress_close ();
    return NULL;
  }

  result = mail_array_create (BOX_MBOX, file);
  atable_point (result->ad);

  while (ret != END_OF_FILE){
    
    ret = mlex_scan ();

    switch (ret){

      case NEXT_MAIL:
        newmail->type = BOX_MBOX;
        mail_array_insert (result, newmail);
        progress_add (mbox_mail_size (newmail));
        break;

      case BROKEN_MAIL:
        fclose (yyin);
        progress_close ();
        mail_array_destroy (result);
        return NULL;
    }
  }
  progress_close ();
  fclose (yyin);
  return result;
}



int
mbox_mail_header (mail_t *mail, char **place, const char *fname)
{
  mime_t *mime = mail->rmime->array[0];
  
  if (file_part (fname, mail->place.offset_header, mime->offset_start,
                 place)){
    *place = NULL;
    return 1;
  }
  return 0;
}



int
mbox_mail_body (mail_t *mail, char **place, mime_t *mime, const char *fname)
{
  if (file_part (fname, mime->offset_start, mime->offset_end, place)){
    *place = NULL;
    return 1;
  }
  return 0;
}



void
mbox_free_resources (void)
{

}

/****************************************************************************
 *    INTERFACE CLASS BODIES
 ****************************************************************************/
/****************************************************************************
 *
 *    END MODULE mbox.c
 *
 ****************************************************************************/
