#ifndef __MIME_H__
#define __MIME_H__ 1
/* 
   elmo - ELectronic Mail Operator

   Copyright (C) 2003 rzyjontko

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

*/
/****************************************************************************
 *    INTERFACE REQUIRED HEADERS
 ****************************************************************************/

#include <stdlib.h>

#include "memchunk.h"

/****************************************************************************
 *    INTERFACE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
 ****************************************************************************/

typedef enum {MTYPE_UNKNOWN,
              MTYPE_TEXT_PLAIN,
              MTYPE_TEXT_OTHER,
              MTYPE_HTML,
              MTYPE_MESSAGE,
              MTYPE_IMAGE,
              MTYPE_APPLICATION,
              MTYPE_X_SOMETHING,
} mime_type_t;
typedef enum {MENC_NONE, MENC_QP, MENC_BASE64} mime_enc_t;

/****************************************************************************
 *    INTERFACE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
 ****************************************************************************/
/****************************************************************************
 *    INTERFACE STRUCTURES / UTILITY CLASSES
 ****************************************************************************/

typedef struct {
  mime_type_t  type;
  mime_enc_t   encoding;
  unsigned     offset_start;
  unsigned     offset_end;
  char        *type_desc;
  char        *file_name;
} mime_t;

/****************************************************************************
 *    INTERFACE DATA DECLARATIONS
 ****************************************************************************/

extern char *mime_enc_str[];
extern char *mime_type_str[];

/****************************************************************************
 *    INTERFACE FUNCTION PROTOTYPES
 ****************************************************************************/

extern char   *mime_decode_header (char *s, int free_s);
extern mime_t *mime_create (mime_type_t type);
extern mime_t *mime_create_from_filename (char *file_name);
extern void    mime_set_from_header (mime_t *mime, char *header);
extern void    mime_complete_file_name (mime_t *mime, char *header);
extern void    mime_destroy (mime_t *mime);
extern char   *mime_encode (char *s, size_t len, size_t *rlen,
                            mime_enc_t encoding);
extern char   *mime_decode (mime_t *mime, char *s, size_t *rlen);
extern char   *mime_get_and_encode (mime_t *mime, size_t *rlen,
                                    const char *fname);
extern void    mime_set_content_type (mime_t *mime, char *type);
extern void    mime_dump (memchunk_t *memchunk, mime_t *mime);
extern mime_t *mime_read (memchunk_t *memchunk);

/****************************************************************************
 *    INTERFACE OBJECT CLASS DEFINITIONS
 ****************************************************************************/
/****************************************************************************
 *    INTERFACE TRAILING HEADERS
 ****************************************************************************/
/****************************************************************************
 *
 *    END HEADER mime.h
 *
 ****************************************************************************/
#endif
