/* 
   elmo - ELectronic Mail Operator

   Copyright (C) 2003 rzyjontko

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

   ----------------------------------------------------------------------

   interactive progress bar
   
*/
/****************************************************************************
 *    IMPLEMENTATION HEADERS
 ****************************************************************************/

#include <stdlib.h>
#include <string.h>

#include "ecurses.h"
#include "screen.h"

/****************************************************************************
 *    IMPLEMENTATION PRIVATE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
 ****************************************************************************/

#define MIN(a,b) ((a)<(b)?(a):(b))

/****************************************************************************
 *    IMPLEMENTATION PRIVATE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE STRUCTURES / UTILITY CLASSES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION REQUIRED EXTERNAL REFERENCES (AVOID)
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE DATA
 ****************************************************************************/

static WINDOW *progress_win = NULL;
static int     maximum      = 0;
static int     current      = 0;


/****************************************************************************
 *    INTERFACE DATA
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTION PROTOTYPES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTIONS
 ****************************************************************************/

static void
draw_progress (void)
{
  int i;
  int cnt;

  if (maximum == 0)
    return;
  
  cnt = 50 * current / maximum;

  wmove (progress_win, 1, 1);
  for (i = 0; i < cnt; i++){
    waddch (progress_win, ' ' | COLOR_PAIR (3));
  }
}


static void
refresh_me (void)
{
  screen_touchwin (progress_win);
  screen_wnoutrefresh (progress_win);
}

/****************************************************************************
 *    INTERFACE FUNCTIONS
 ****************************************************************************/


void
progress_open (char *title)
{
  int xpos;
  int len = MIN (strlen (title), 50);

  progress_win = newwin (3, 52, (LINES - 5) / 2, (COLS - 52) / 2);
  wborder (progress_win, 0, 0, ' ', 0, ' ', ' ', 0, 0);
  xpos = (52 - len) / 2 - 2;
  mvwinsch (progress_win, 0, 0, ACS_ULCORNER);
  mvwhline (progress_win, 0, 1, ACS_HLINE, xpos - 1);
  mvwinsch (progress_win, 0, xpos, ACS_RTEE);
  wmove (progress_win, 0, xpos + 1);
  waddstr (progress_win, title);
  winsch (progress_win, ACS_LTEE);
  mvwhline (progress_win, 0, xpos + len + 2, ACS_HLINE,
            52 - (xpos + len + 3));
  mvwinsch (progress_win, 0, 52 - 1, ACS_URCORNER);

  current = 0;
  maximum = 0;

  refresh_me ();
}



void
progress_set_max (int max)
{
  maximum = max;
}


void
progress_add (int count)
{
  int newcurrent = MIN (current + count, maximum);
  
  if (progress_win == NULL)
    return;
  
  if (newcurrent > current){
    current = newcurrent;
    draw_progress ();
    refresh_me ();
  }
}


void
progress_close (void)
{
  if (progress_win == NULL)
    return;
  
  current = maximum;
  draw_progress ();
  refresh_me ();
  delwin (progress_win);
}



/****************************************************************************
 *    INTERFACE CLASS BODIES
 ****************************************************************************/
/****************************************************************************
 *
 *    END MODULE progress.c
 *
 ****************************************************************************/
