#ifndef __SELECT_H__
#define __SELECT_H__ 1
/* 
   elmo - ELectronic Mail Operator

   Copyright (C) 2003 rzyjontko

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

*/
/*****************************************************************************
 *    INTERFACE REQUIRED HEADERS
 ****************************************************************************/

#include "ecurses.h"
#include "rstring.h"
#include "bitarray.h"

/*****************************************************************************
 *    INTERFACE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
 ****************************************************************************/
/****************************************************************************
 *    INTERFACE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
 ****************************************************************************/
/****************************************************************************
 *    INTERFACE STRUCTURES / UTILITY CLASSES
 ****************************************************************************/

/**
 * win      -- selection window (used also to get coordinates and size)
 * bar      -- just a bar
 * items    --
 * top_pos  -- index of item which is at the top of window
 * bar_pos  -- selected item index
 * with_box -- 1 - frame box, 0 - no frame box
 * no_bar   -- 1 - no bar and automatic scrolling
 */
typedef struct select {
  WINDOW     *win;
  WINDOW     *bar;
  rstring_t  *items;
  bitarray_t *selected;
  int         top_pos;
  int         bar_pos;
  int         with_box;
  int         no_bar;
} select_t;

/****************************************************************************
 *    INTERFACE DATA DECLARATIONS
 ****************************************************************************/
/****************************************************************************
 *    INTERFACE FUNCTION PROTOTYPES
 ****************************************************************************/

extern select_t *select_open (rstring_t *items, int w, int h, int with_box,
                              int no_bar, const char *title);
extern void      select_move_vert (select_t *select, int newy);
extern int       select_close (select_t *select);
extern void      select_show (select_t *select);
extern void      select_next (select_t *select);
extern void      select_prev (select_t *select);
extern void      select_next_page (select_t *select);
extern void      select_prev_page (select_t *select);
extern void      select_first (select_t *select);
extern void      select_last (select_t *select);
extern void      select_change (select_t *select);
extern void      select_change_all (select_t *select);
extern void      select_set (select_t *select);
extern void      select_unset (select_t *select);
extern void      select_set_all (select_t *select);
extern void      select_unset_all (select_t *select);
extern void      select_redraw (select_t *select);
extern void      select_add_item (select_t *select, char *item);
extern void      select_remove_item (select_t *select, int index);
extern void      select_remove_selected (select_t *select);
extern void      select_install_new_items (select_t *select, rstring_t *item);

/****************************************************************************
 *    INTERFACE OBJECT CLASS DEFINITIONS
 ****************************************************************************/
/****************************************************************************
 *    INTERFACE TRAILING HEADERS
 ****************************************************************************/
/****************************************************************************
 *
 *    END HEADER select.h
 *
 ****************************************************************************/
#endif
