/* 
   elmo - ELectronic Mail Operator

   Copyright (C) 2002 rzyjontko

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

   ----------------------------------------------------------------------

   status line interface
   
*/
/****************************************************************************
 *    IMPLEMENTATION HEADERS
 ****************************************************************************/

#include <string.h>

#include "ecurses.h"
#include "xmalloc.h"
#include "status.h"
#include "line.h"
#include "screen.h"
#include "gettext.h"

/****************************************************************************
 *    IMPLEMENTATION PRIVATE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE STRUCTURES / UTILITY CLASSES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION REQUIRED EXTERNAL REFERENCES (AVOID)
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE DATA
 ****************************************************************************/

static WINDOW *statusbar   = NULL;
static line_t *status_line = NULL;

/****************************************************************************
 *    INTERFACE DATA
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTION PROTOTYPES
 ****************************************************************************/

static void change_me (void);

/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTIONS
 ****************************************************************************/

static void
change_me (void)
{
  mvwaddstr (statusbar, 0, 0, status_line->str);
  screen_wnoutrefresh (statusbar);
}


/****************************************************************************
 *    INTERFACE FUNCTIONS
 ****************************************************************************/

void
status_start (void)
{
  statusbar = newwin (1, COLS - 13, LINES - 2, 0);
  wbkgdset (statusbar, COLOR_PAIR (1));

  leaveok (statusbar, TRUE);

  status_line = line_create (COLS - 12);
  
  change_me ();
}



void
status_free_resources (void)
{
  if (status_line){
    line_destroy (status_line);
    status_line = NULL;
    delwin (statusbar);
  }
}




void
status_put_str_len (const char *str, size_t len)
{
  line_put_str_len (status_line, str, len);
  change_me ();
}



void
status_put_str (const char *str)
{
  line_put_str (status_line, str);
  change_me ();
}



void
status_put_mailbox (const char *name, int total, int old, int new)
{
  line_put_str (status_line, "   ");
  line_add_str (status_line, name);
  line_add_str (status_line, _("  [msgs:"));
  line_add_int (status_line, total);
  if (old){
    line_add_str (status_line, _("  old:"));
    line_add_int (status_line, old);
  }
  if (new){
    line_add_str (status_line, _("  new:"));
    line_add_int (status_line, new);
  }
  line_add_str (status_line, "]");
  change_me ();
}



void
status_put_mailinfo (const char *from, const char *subject)
{
  line_put_str (status_line, "   ");
  if (from)
    line_add_str (status_line, from);
  line_add_str (status_line, "      ");
  if (subject)
    line_add_str_dots (status_line, subject);
  change_me ();
}



void
status_refresh (void)
{
  if (status_line)
    change_me ();
}


/****************************************************************************
 *    INTERFACE CLASS BODIES
 ****************************************************************************/
/****************************************************************************
 *
 *    END MODULE status.c
 *
 ****************************************************************************/
