/* 
   elmo - ELectronic Mail Operator

   Copyright (C) 2003 rzyjontko

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

   ----------------------------------------------------------------------

   resizable strings

*/
/****************************************************************************
 *    IMPLEMENTATION HEADERS
 ****************************************************************************/

#include <stdarg.h>
#include <stdio.h>
#include <string.h>

#include "str.h"
#include "xmalloc.h"

/****************************************************************************
 *    IMPLEMENTATION PRIVATE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
 ****************************************************************************/

#define INITIAL_SIZE 64

/****************************************************************************
 *    IMPLEMENTATION PRIVATE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE STRUCTURES / UTILITY CLASSES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION REQUIRED EXTERNAL REFERENCES (AVOID)
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE DATA
 ****************************************************************************/
/****************************************************************************
 *    INTERFACE DATA
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTION PROTOTYPES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTIONS
 ****************************************************************************/
/****************************************************************************
 *    INTERFACE FUNCTIONS
 ****************************************************************************/


str_t *
str_create_size (int size)
{
  str_t *result = xmalloc (sizeof (str_t));

  result->len  = 0;
  result->size = size;
  result->str  = xmalloc (size + 1);
  *result->str = '\0';

  return result;
}


str_t *
str_create (void)
{
  return str_create_size (INITIAL_SIZE);
}


void
str_destroy (str_t *str)
{
  if (str->str)
    xfree (str->str);
  xfree (str);
}



char *
str_finished (str_t *str)
{
  char *result = xrealloc (str->str, str->len + 1);

  xfree (str);
  return result;
}


void
str_putchar (str_t *str, int c)
{
  if (str->len >= str->size - 1){
    str->size *= 2;
    str->str   = xrealloc (str->str, str->size + 1);
  }
  str->str[str->len]     = c;
  str->str[str->len + 1] = '\0';
  str->len++;
}


int
str_vsprintf (str_t *str, const char *fmt, va_list ap)
{
  int     n;
  int     max_size;
  va_list copy;
  
  while (1){
    max_size = str->size - str->len - 1;
    memcpy (&copy, &ap, sizeof (va_list));
    n = vsnprintf (str->str + str->len, max_size, fmt, copy);
    if (n > -1 && n < max_size){
      str->len += n;
      return n;
    }
    if (n > -1)
      while (max_size < n + 1){
        str->size *= 2;
        max_size   = str->size - str->len;
      }
    else {
      str->size *= 2;
    }

    str->str = xrealloc (str->str, str->size + 1);
  }
}


int
str_sprintf (str_t *str, const char *fmt, ...)
{
  int     ret;
  va_list ap;

  va_start (ap, fmt);
  ret = str_vsprintf (str, fmt, ap);
  va_end (ap);
  return ret;
}

/****************************************************************************
 *    INTERFACE CLASS BODIES
 ****************************************************************************/
/****************************************************************************
 *
 *    END MODULE str.c
 *
 ****************************************************************************/
